////////////////////////////////////////////////////////////////////////////////
///
///		Title		: CNPUnzipFile Class
///
///		Copyright (c) 2007, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: ũ   
///		Author		: oswalk
///		Author(TO)	: 
///		Last-Update	: 2007-02-08
///
////////////////////////////////////////////////////////////////////////////////
#pragma once

/**
* @group 	PCM 
* @brief 	{kr}  {/kr}
* @date		02/08/2007
* @author	{kr} oswalk {/kr}
* @remark	
*/
#include <atlcoll.h>
#include "../zlib/zlib.h"
#include "NPZipErrorCode.h"
#include <vector> 
#define MAX_DIRECTORY 512
struct UnzipGlobalInfo
{
	uLong m_lNumberEntry;
	uLong m_lCommentSize;
};


struct UnzipFileInfo
{
    uLong m_lVersion;				/* version made by                 2 bytes */
    uLong m_lversionExtract;		/* version needed to extract       2 bytes */
    uLong m_lFlag;					/* general purpose bit flag        2 bytes */
    uLong m_lCompressionMethod;		/* compression method              2 bytes */
    uLong m_lDosDate;				/* last mod file date in Dos fmt   4 bytes */
    uLong m_lCrc;					/* crc-32                          4 bytes */
    uLong m_lCompressedSize;		/* compressed size                 4 bytes */ 
    uLong m_lUncompressedSize;		/* uncompressed size               4 bytes */ 
    uLong m_lFilenameSize;			/* filename length                 2 bytes */
    uLong m_lFileExtraSize;			/* extra field length              2 bytes */
    uLong m_lFileCommentSize;		/* file comment length             2 bytes */

    uLong m_lDiskNumStart;			/* disk number start               2 bytes */
    uLong m_lInternal_FA;			/* internal file attributes        2 bytes */
    uLong m_lExternal_FA;			/* external file attributes        4 bytes */

    //CTime m_Tmu_date;
};



struct UnzipFileInfo_Internal
{
    uLong m_lCurrentFileOffset;			/* relative offset		4 bytes */
};


struct ReadFileInfoInZip
{
	ReadFileInfoInZip();
	~ReadFileInfoInZip();
	BYTE  * m_pReadBuffer;				/* internal buffer for compressed data */
	z_stream m_Stream;					/* zLib stream structure for inflate */
	
	uLong m_lPosInZipFile;				/* position in byte on the zipfile, for fseek*/
	uLong m_lStreamFlag;				/* flag set if stream structure is initialised*/
	
	uLong m_lExtraOffset;				/* offset of the local extra field */
	uInt  m_lExtraSize;					/* size of the local extra field */
	uLong m_lPosExtraField;				/* position in the local extra field in read*/
	
	uLong m_lCrc32;						/* m_lCrc32 of all data uncompressed */
	uLong m_lCrc32_wait;				/* m_lCrc32_wait we must obtain after decompress all */
	uLong m_lRestReadCompressed;		/* number of byte to be decompressed */
	uLong m_lRestReadUncompressed;		/*number of byte to be obtained after decomp*/
	
	uLong m_lCompressionMethod;			/* compression method (0==store) */
	uLong m_lUnzipFileSize;				/* byte before the zipfile, (>0 for sfx)*/
};


struct UnzipInfo
{
	UnzipInfo();
	~UnzipInfo();
	UnzipGlobalInfo gi;				
	uLong	m_lUnzipFileSize;	
	uLong	m_lFileNum;					
	uLong	m_lPosCentralDir;		
	uLong	m_lPosCentral;				
	
	uLong	m_lCentralDirSize;	
	uLong	m_lCentralDirOffset;
	BOOL	m_lFileOK;			
							
		
	UnzipFileInfo			m_FileInfo;	
	UnzipFileInfo_Internal	m_FileInfoInternal;
    ReadFileInfoInZip	*	m_pCurrentFileInfoInZip;
	
	void Free_CurrentFileInfoInZip();
	void Alloc_CurrentFileInfoInZip();
};

struct DirectoryInfo
{
public:
	DirectoryInfo();
	~DirectoryInfo();
	char	m_szDirectory[MAX_DIRECTORY];
	DirectoryInfo * GetParentDiretory();
	void			SetParentDiretory(DirectoryInfo * pParentDirectory);
	DirectoryInfo * GetChildDirectory(int nIndex);
	int				GetChildDirectoryCount();
	BOOL			InsertChildDirectory(LPCSTR pszcDirctory);
private:	
	std::vector<DirectoryInfo> m_vChildDirectory;
	DirectoryInfo *	m_pParentDirectory;			
};

class CNPVector;
class INPZipData;
class CNPUnzipFile
{
public:
	CNPUnzipFile();
	virtual		~CNPUnzipFile();
	/**
	* @brief	{kr} zip  .{/kr}
	* @param	pZipData		{kr}[in] zip͸ ϴ ̽{/kr}
	*/
	void		Open(INPZipData * pZipData);
	
	/**
	* @brief	{kr} Zip Ͽ ִ   (ϵ) д´.{/kr}
	* @param	pFileInfo				{kr} [out]  {/kr}
	* @param	szFileName				{kr} [out] ̸{/kr}
	* @param	lFileNameBufferSize		{kr} [in] ̸  ũ{/kr}
	* @param	pExtraField				{kr} [out]빮  {/kr}
	* @param	lExtraFieldBufferSize	{kr} [in] ũ{/kr}
	* @param	szComment				{kr} [out]zipϿ comment{/kr}
	* @param	lCommentBufferSize		{kr} [in] ũ{/kr}
	* @return	{kr}  θ Ѵ.{/kr}
	*/
	int			GetCurrentFileInfo( 
										UnzipFileInfo* pFileInfo, 
										LPSTR	szFileName = NULL, 
										uLong	lFileNameBufferSize = 0, 
										void	*pExtraField = NULL, 
										uLong	lExtraFieldBufferSize = 0,
										LPSTR	szComment = NULL, 
										uLong	lCommentBufferSize = 0
								  );
	/**
	* @brief	{kr} 丮  ´.{/kr}
	* @param	pDirectoryInfo		{kr} [out]丮 {/kr}
	* @return	{kr}  θ Ѵ.{/kr}
	*/
	BOOL		GetDirectoryInfo(
									DirectoryInfo * pDirectoryInfo
								);
	/**
	* @brief	{kr} ù° Ϸ ġ ̵Ѵ.{/kr}
	*/
	void		GoToFirstFile();
	
	/**
	* @brief	{kr}  Ϸ ġ ̵Ѵ.{/kr}
	* @return	{kr} ̵ θ Ѵ.{/kr}
	*/
	BOOL		GoToNextFile();
	
	/**
	* @brief	{kr} zip Ͽ ϴ  ã,   ͸ Ѵ.{/kr}
	* @param	szFileName		{kr}  ̸{/kr}
	* @param	bCaseSensitive		{kr} 빮  {/kr}
	* @return	{kr}    {/kr}
	*/
	int			FindFileInZip(LPSTR szFileName, BOOL bCaseSensitive = FALSE);
	
	/**
	* @brief	{kr} Zip Ͽ    д´.{/kr}
	* @return	{kr}  ߻ θ {/kr}
	*/
	int			OpenCurrentFile();
	
	/**
	* @brief	{kr} Zip Ͽ    ͸ Ѵ.{/kr}	
	* @return	{kr}  ߻ θ {/kr}
	*/
	int			CloseCurrentFile();
	
	/**
	* @brief	{kr}  ũ ŭ   ͸ д´.{/kr}
	* @param	pBuffer		{kr}[out]  {/kr}
	* @param	nBufferSize		{kr}[in]  ũ{/kr}
	* @return	{kr}   {/kr}
	*/
	int			ReadCurrentFile(void* pBuffer, UINT nBufferSize);

	/**
	* @brief	{kr} zip  ݴ´/{/kr}
	*/
	void		Close();	
	
	/**
	* @brief	{kr} Ư Directory ã{/kr}
	* @param	pDirectoryInfo		{kr} [in]丮 {/kr}
	* @param	pszcDirectory		{kr} [in]ã 丮{/kr}
	* @return	{kr} pDirectoryInfo pszcDirectory   {/kr}
	*/
	DirectoryInfo *	FindDirectory(DirectoryInfo * pDirectoryInfo, LPSTR pszcDirectory);

protected:
	/**
	* @brief	{kr} Zip Ͽ ִ   (ϵ) д´.{/kr}
	* @param	pFileInfo				{kr} [out]  {/kr}
	* @param	pFileInfoInternal		{kr} [out]  {/kr}
	* @param	szFileName				{kr} [out] ̸{/kr}
	* @param	lFileNameBufferSize		{kr} [in] ̸  ũ{/kr}
	* @param	pExtraField				{kr} [out]빮  {/kr}
	* @param	lExtraFieldBufferSize	{kr} [in] ũ{/kr}
	* @param	szComment				{kr} [out]zipϿ comment{/kr}
	* @param	lCommentBufferSize		{kr} [in] ũ{/kr}
	* @return	{kr}  θ Ѵ.{/kr}
	*/
	int			GetCurrentFileInfoInternal( 
												UnzipFileInfo & pFileinfo, 
												UnzipFileInfo_Internal & pFileInfoInternal, 
												LPSTR	szFileName,
												uLong	lFileNameBufferSize, 
												void	*lExtraField, 
												uLong	lExtraFieldBufferSize,
												LPSTR	szComment, 
												uLong	lCommentBufferSize
										  );
	
	/**
	* @brief	{kr} Dos ð/¥ CTime ȯ{/kr}
	*/
	void		DosDateToTmuDate(UnzipFileInfo & file_info);
	
	/**
	* @brief	{kr}   о zip   θ ȮѴ.{/kr}
	*/
	int			CheckCurrentFileCoherencyHeader (	
													uInt & nSizeVar,
													uLong & lOffset_local_extrafield,
													uLong & lSize_local_extrafield
												);
	
	/**
	* @brief	{kr} zip   global comment  ġѴ.{/kr}
	* @return	{kr}  ߻ θ {/kr}
	*/
	uLong		SearchCentralDir();
	
	/**
	* @brief	{kr} 4bytes ͸ д´.{/kr}
	* @param	lLong		{kr}[out] ͸  {/kr}
	*/
	void		GetLong (uLong & lLong);
	
	/**
	* @brief	{kr} 2bytes ͸ д´.{/kr}
	* @param	lLong		{kr}[out] ͸  {/kr}
	*/
	void		GetShort (uLong & lLong);
	
	/**
	* @brief	{kr} 1bytes ͸ д´.{/kr}
	* @param	nInt		{kr}[out] ͸  {/kr}
	*/
	void		GetByte(int & nInt);
	
	/**
	* @brief	{kr}  ̸ Ѵ.{/kr}
	* @param	strFileName1		{kr}  ̸{/kr}
	* @param	strFileName2		{kr}  ̸{/kr}
	* @param	bCaseSensitive		{kr} 빮  {/kr}
	* @return	{kr} ̸   {/kr}
	*/
	int			StringFileNameCompare(LPSTR strFileName1, LPSTR strFileName2, BOOL bCaseSensitive = FALSE);
	void		InsertDirectory(DirectoryInfo * pDirectoryInfo, LPSTR pszcDirectory);
	static void		MyFree(void* opaque, void* address);
	static void*	MyAlloc(void* opaque, UINT items, UINT size);
private:
	UnzipInfo uf;
	INPZipData * m_pZipData;
	
	CAtlList<void *> m_list;
	BOOL m_bDetectZlibMemoryLeaks;
};


